/*******************************************************************************
*
*      Company                   :  CAP GEMINI
*      Service Line              :  Product Devlopment
*      Project                   :  FingerChip
*      Task                      :  
*
*      Soft name                 :  $RCSfile: trainingdlg.cpp,v $
*      Soft version              :  $Revision: 1.3 $
*      Last modified             :  $Date: 2003/07/17 11:00:51 $
*
*      Description               :  This is the main dialog  of the application
*
*      Authors                   :  Thomasson Nicolas
*
*      Platform                  :  PC
*      System                    :  Win 9x, NT, 2K
*
*      Operating System          :  Windows
*      Date of creation          :  01/30/2001
*
*      Prefix                    :
*      Code size (.o) (KB)       :
*
*      References				 :
* 
*      History			          
*
*         04/26/2001 THM         : Added OnDestroy message handler to prevent
*                                  ESC bug.
*
*         05/03/2001 THM         : Added specific error message for FC_BUSY
*
*		  06/27/2001 THM		 : ScreenSaver bug corrected
*								   Added exe Copyright
*
*     Copyright (c) 
*     All rights reserved
*
******************************************************************************/
#pragma comment(exestr,"Copyright (c) 1998-2001 Atmel-Grenoble. All rights reserved. \
Warning: This computer program is protected by copyright law and \
international treaties. This computer program has been provided for \
demonstration purpose only and is provided on an 'AS IS' basis \
with no warranty whatsoever.")

#include "../include/stdafx.h"
#include "../include/Training.h"
#include "../include/TrainingDlg.h"
#include "../include/help.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

BOOL m_bReturnFromSuspend ;

/******************************************************************************
*
* CTrainingDlg::CTrainingDlg
*
* Prototype		:	CTrainingDlg::CTrainingDlg(CWnd* pParent)
*
* Description	:	Constructor for CTrainingDlg class
*					
* In			:	pParent :
*
* PreCond		:	None
*
* Out			:	None
*
* PostCond		:	None
*
* Remarks		:	None
*
******************************************************************************/
CTrainingDlg::CTrainingDlg(CWnd* pParent)
	: CDialog(CTrainingDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CTrainingDlg)
	m_StStatusValue = _T("");
	m_StBandwidthValue = _T("");
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32	
	killAcquisitionThread = false;
	m_bReturnFromSuspend = FALSE ;
}


/******************************************************************************
*
* CTrainingDlg::DoDataExchange
*
* Prototype		:	CTrainingDlg::DoDataExchange(data update)
*
* Description	:	Constructor for CTrainingDlg class
*					
* In			:	update : object to suuports dialog data exchange
*
* PreCond		:	see MSDN CDataExchange
*
* Out			:	None
*
* PostCond		:	see MSDN CDataExchange
*
* Remarks		:	None
*
******************************************************************************/
void CTrainingDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CTrainingDlg)
	DDX_Control(pDX, IDC_ST_BANDWIDTH, m_StBandwidthCtrl);
	DDX_Control(pDX, IDC_FINGER, m_finger);
	DDX_Text(pDX, IDC_ST_STATUS, m_StStatusValue);
	DDX_Text(pDX, IDC_ST_BANDWIDTH, m_StBandwidthValue);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CTrainingDlg, CDialog)
	//{{AFX_MSG_MAP(CTrainingDlg)
	ON_WM_CLOSE()
	ON_COMMAND(ID_FILE_QUIT, OnFileQuit)
	ON_COMMAND(ID_HELP_ABOUT, OnHelpAbout)
	ON_COMMAND(ID_HELP_CONTENT, OnHelpContent)
	ON_WM_DESTROY()
	ON_COMMAND(ID_README, OnReadme)
	//}}AFX_MSG_MAP
	ON_COMMAND(1016,NewImage)
END_MESSAGE_MAP()


/******************************************************************************
*
* CTrainingDlg::OnInitDialog
*
* Prototype		:	CTrainingDlg::OnInitDialog(event)
*
* Description	:	CTrainingDlg message handlers
*					
* In			:	event
*
* PreCond		:	the dialog is about to be displayed
*
* Out			:	None
*
* PostCond		:	None
*
* Remarks		:	None
*
******************************************************************************/

BOOL CTrainingDlg::OnInitDialog()
{
	int					returnCode;				
	char				errorToDisplay[256];
	unsigned char		message[256];
	t_FC_DeviceInfo		DeviceInfo;

	returnCode = FC_OK;	

	// Event to order the call to Fc_GetImage API
	hEvent=CreateEvent(0,false,false,"");

	CDialog::OnInitDialog();
	// Set the icon for this dialog.  The framework does this automatically
	// when the application's main window is not a dialog
	HICON m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
	SetIcon(m_hIcon, TRUE);				// Set big icon
	SetIcon(m_hIcon, FALSE);			// Set small icon
	
	// Intialize the error to display
	strcpy(errorToDisplay,"");

	// Initialize the debug file if needed
#ifdef GENERATE_DEBUG_FILE
	//Using errorToDisplay buffer to store the current directory.
	returnCode=GetCurrentDirectory(256,errorToDisplay);
	if(returnCode)
	 {
		strcpy(errorToDisplay+returnCode,"\\debugfile.txt");
		returnCode=FC_DebugFileGeneration(FC_LOG_ENABLE_ALWAYS|FC_LOG_ERASE,errorToDisplay);		
	 }
	if(returnCode)	
		::MessageBox(NULL,"Error during the initialisation. \n"
							"Can't open the debug file !","",
							MB_ICONHAND | MB_OK);
#endif
	
	// Getting device information to fill the about dialog box
	if(FC_GetDeviceInfo(&DeviceInfo) == FC_OK)
	{
		DlgAbout.m_StMessageValue.Format(
			"DLL version : %s\nPart number : %s\n\n%s"
			,DeviceInfo.dllVersion,DeviceInfo.partNumber,DeviceInfo.copyright);
	}
	else
	{
		DlgAbout.m_StMessageValue="An error occured retrieving\n device information";
	}

	// Call the FC_OpenDevice function from the Dll
	returnCode = FC_OpenDevice();

	// Error processing 
	if (returnCode < 0)
	{
		FC_FormatMessage(returnCode, message, 256); 
		
		m_StStatusValue.Format("Error %s  during hardware initialisation",
								message);
		if(returnCode==FC_BUSY)
			::MessageBox(NULL,"Error during the initialisation. \n"
							"An application seems to use the FingerChip already",
							"FingerChip Training application error",
							MB_ICONHAND | MB_OK);
			else
			::MessageBox(NULL,(const char*)message, "FingerChip Training application error", MB_ICONHAND | MB_OK);
		pBitmap=NULL;
		OnClose();
		return FALSE;
	}
	else
	{		
		//Retrieve bandwidth information and update specific display
		if(FC_GetDeviceInfo(&DeviceInfo) == FC_OK)
		{
			//Enable or disable the bandwidth information depending on device type
			m_StBandwidthCtrl.EnableWindow( DeviceInfo.communicationPort == USB );
			m_StBandwidthValue = "USB Bandwidth\n";
			if( DeviceInfo.communicationPort == USB )
			{
				switch( DeviceInfo.realBandwidth )
				{
					case REALBWNo: m_StBandwidthValue += "0%"; break;
					case REALBWHalf: m_StBandwidthValue += "50%"; break;
					case REALBWFull: m_StBandwidthValue += "100%"; break;
				}
			}
			else
				m_StBandwidthValue += "not applicable";
		}
	
		if( returnCode == FC_OK)	
			m_StStatusValue.Format(
				"Correct hardware initialisation. \n Waiting for a finger...");
		else 	//On warning
		{
			//For half bandwidth in automatic mode, display message box in addition
		if(returnCode == FC_HALF_BANDWIDTH)
			::MessageBox(NULL,"Warning during the initialisation. \n"
							"Only half of the USB bandwidth is available. Sweep your finger slowly or unplug other USB periperals",
							"FingerChip Training application warning",
							MB_ICONHAND | MB_OK);

			FC_FormatMessage(returnCode, message, 256); 
			m_StStatusValue.Format(
				"Warning %i during hardware initialisation - %s",returnCode,message);
		}
		
		// display the message	
		UpdateData(FALSE);	

		//Memory allocation for a bitmap
		pBitmap=new unsigned char[sizeof(BITMAPINFOHEADER) 
					+ sizeof(RGBQUAD) * 256 + XImage*YImage];

		//Pointer into the bitmap to the palette
		unsigned char *GreyPal= (BYTE *) pBitmap + sizeof(BITMAPINFOHEADER);

		//Image data (1 byte per pixel) follow the palette into the bitmap
		pImage= GreyPal + sizeof(RGBQUAD) * 256;

		// Fill the bitmap header
		BITMAPINFOHEADER *bmi	= (BITMAPINFOHEADER*)pBitmap;
		bmi->biSize				= sizeof(BITMAPINFOHEADER);
		bmi->biWidth			= XImage;
		bmi->biHeight			= YImage;
		bmi->biPlanes			= 1;
		bmi->biBitCount			= 8; 
		bmi->biCompression		= BI_RGB;
		bmi->biSizeImage		= 0; 
	    bmi->biXPelsPerMeter	=0;
	    bmi->biYPelsPerMeter	=0;
	    bmi->biClrUsed			= 0;
	    bmi->biClrImportant		= 0;
	    
		// Bitmap palette: grey scale, 256 levels are created
		CreateGrayLevelPal(GreyPal);			

		//use to stop the acquisition thread
		hMutex=CreateMutex(0,true,"");
		
		//Create the thread.
		AfxBeginThread(Threading, this);
	}

	// display the message
	UpdateData(FALSE); 
		
	// return TRUE  unless you set the focus to a control
	return TRUE;  
}
/* end of CTrainingDlg::OnInitDialog */

/******************************************************************************
*
* CTrainingDlg::NewImage
*
* Prototype		:	CTrainingDlg::NewImage(short errorCode)
*
* Description	:	If the errorCode is correct, display the image.
*					
* In			:	errorCode : error code
*
* PreCond		:	None
*
* Out			:	None
*
* PostCond		:	None
*
* Remarks		:	None
*
******************************************************************************/
void CTrainingDlg::NewImage(short errorCode) 
{
	unsigned long	TimeOut;
	unsigned char	message[256];
	int				timeScreenSaver;
	HWND			wnd;


	TimeOut = 0;

	// We call directly the FC_GetImage API, control will be given back only 
	// if a finger is swept or timeout
	UpdateData(true);

	if(!( (errorCode == FC_TIME_OUT) || (errorCode < FC_OK)) )
	{		

		m_StStatusValue = "Acquisition finished. Reconstructing...";		

		// Find and release the screen saver if necessary
		wnd=::FindWindow("WindowsScreenSaverClass", NULL);

		if(wnd)
		::SendMessage(wnd,WM_CLOSE,0,0);

		// Reinitialize screensaver 
		SystemParametersInfo(SPI_GETSCREENSAVETIMEOUT,
									0,
									&timeScreenSaver,
									0);

		SystemParametersInfo(SPI_SETSCREENSAVETIMEOUT,
									timeScreenSaver,
									NULL,
									0);

		// process the data
		// here, we just display the fingerprint. In a real application,
		// authentication must be performed now.
		CDC *pDC=m_finger.GetDC();
		::SetDIBitsToDevice(pDC->m_hDC,					// hDC
							0,0,					// DestX,DestY
							imageInfo.xReturnSize,imageInfo.yReturnSize,
							0,0,					// SrcX,SrcY
							0,						// nStartScan
							(WORD)imageInfo.yReturnSize,
							pImage,					// lpBits
							(LPBITMAPINFO)pBitmap,	// lpBitsInfo
							DIB_RGB_COLORS);		// wUsage


		UpdateData(FALSE); // display the message on the screen
	}

	if(!(errorCode == FC_TIME_OUT))
	{
		// If FC_TIME_OUT do not modify the text
		FC_FormatMessage(errorCode,message,256);
		m_StStatusValue.Format("%s\nWaiting for a finger...",message);	
		UpdateData(FALSE); // display the message on the screen
	}
}
/* end of CTrainingDlg::NewImage */


/******************************************************************************
*
* CTrainingDlg::OnClose
*
* Prototype		:	CTrainingDlg::OnClose()
*
* Description	:	Call the FC_Close API from the Dll to release the access 
*					to the device
* In			:	None
*
* PreCond		:	None
*
* Out			:	None
*
* PostCond		:	None
*
* Remarks		:	None
*
******************************************************************************/
void CTrainingDlg::OnClose() 
{
	killAcquisitionThread = true;
	SetEvent(hEvent);
	CDialog::OnOK();
	
}


/******************************************************************************
*
* CTrainingDlg::OnFileQuit
*
* Prototype		:	CTrainingDlg::OnFileQuit()
*
* Description	:	The user closes the application with the file menu. The 
*					OnClose method is called
*
* In			:	None
*
* PreCond		:	None
*
* Out			:	None
*
* PostCond		:	None
*
* Remarks		:	None
*
******************************************************************************/
void CTrainingDlg::OnFileQuit() 
{
	OnClose();		
}


/******************************************************************************
*
* CTrainingDlg::CreateGrayLevelPal
*
* Prototype		:	CTrainingDlg::CreateGrayLevelPal(unsigned char * pal)
*
* Description	:	the palette is filled with gray levels
*
* In			:	None
*
* PreCond		:	palette must be allocated
*
* Out			:	pal : pointer on the palette to fill
*
* PostCond		:	None
*
* Remarks		:	None
*
******************************************************************************/
void CTrainingDlg::CreateGrayLevelPal(unsigned char * pal)
{
	int i;
	for(i=0;i<=255;i++)
		{*pal++=i; *pal++=i; *pal++=i; *pal++=0; }
}


/******************************************************************************
*
* CTrainingDlg::OnHelpAbout
*
* Prototype		:	CTrainingDlg::OnHelpAbout()
*
* Description	:	the user querry the about dialog box from the file menu
*
* In			:	None
*
* PreCond		:	None
*
* Out			:	None
*
* PostCond		:	None
*
* Remarks		:	None
*
******************************************************************************/
void CTrainingDlg::OnHelpAbout()
{
	DlgAbout.DoModal();
}


/******************************************************************************
*
* CTrainingDlg::Threading
*
* Prototype		:	CTrainingDlg::Threading(LPVOID pParam)
*
* Description	:	This method can be called in parameter of a CreateThread
*					The method is static and allow to call the Threading() 
*					method. 
*
* In			:	pParam : Pointer on the CTrainingDlg instance
*
* PreCond		:	None
*
* Out			:	None
*
* PostCond		:	None
*
* Remarks		:	None
*
******************************************************************************/
UINT CTrainingDlg::Threading(LPVOID pParam)
{
	CTrainingDlg * myIntance;
	
	myIntance = (CTrainingDlg *)pParam;
    myIntance->Threading();

	return 0;
	
}


/******************************************************************************
*
* CTrainingDlg::Threading
*
* Prototype		:	CTrainingDlg::Threading()
*
* Description	:	This method call the FC_GetImage API from the Dll and send
*					a message when treatment is ended
*
* In			:	None
*
* PreCond		:	None
*
* Out			:	None
*
* PostCond		:	None
*
* Remarks		:	None
*
******************************************************************************/
void CTrainingDlg::Threading()
{
	short criticalErrorNbr = 0;
	short returnValue, retVal;

	while( !killAcquisitionThread)
	{
		returnValue=FC_GetImage(pImage,XImage,YImage,&imageInfo);

		if(WaitForSingleObject(hMutex,0)==WAIT_OBJECT_0)
		{
			CloseHandle(hMutex);
			hMutex = 0 ;
			return;
		}
		PostMessage(WM_COMMAND,UPDATE_IMAGE,returnValue);
		WaitForSingleObject(hEvent,INFINITE);
		if(!killAcquisitionThread && (	returnValue==FC_DRIVER_ERROR ||
										returnValue==FC_DEVICE_NOT_CONNECTED ||
										returnValue==FC_DRIVER_NOT_FOUND ||
										returnValue==FC_ERROR ||
										returnValue==FC_NOT_OPEN))
		{
			// It may be an ESD recovery
			PostMessage(WM_COMMAND,HOT_DETACH,0);	// to update message area
			LogMessage("Launching FC_CloseDevice then FC_OpenDevice\r\n");
			FC_CloseDevice();
			Sleep(500);
			retVal=FC_OpenDevice();
			if(retVal >= 0)
				PostMessage(WM_COMMAND,HOT_ATTACH,0);	// to update message area
		}
	}
}


/******************************************************************************
*
* CTrainingDlg::OnCommand
*
* Prototype		:	CTrainingDlg::OnCommand(WPARAM wParam, LPARAM lParam)
*
* Description	:	When UPDATE_MESSAGE is received, Call NewImage to display
*					and set the event to order the thread to read another finger
*
* In			:	wParam : Message type
*					lParam : Parameter to transmit
*
* PreCond		:	None
*
* Out			:	None
*
* PostCond		:	None
*
* Remarks		:	None
*
******************************************************************************/
BOOL CTrainingDlg::OnCommand(WPARAM wParam, LPARAM lParam) 
{
	if(wParam==UPDATE_IMAGE)
	{
		NewImage((short)lParam);
		SetEvent(hEvent);
		return true;
	}	
	if(wParam==HOT_DETACH)
	{
		m_StStatusValue.Format("Device is detached.\nTrying to open it...");	
		UpdateData(FALSE); // display the message on the screen
		return true;
	}	
	if(wParam==HOT_ATTACH)
	{
		m_StStatusValue.Format("Device is now opened.\nWaiting for a finger...");	
		UpdateData(FALSE); // display the message on the screen
		return true;
	}	
			
	return CDialog::OnCommand(wParam, lParam);
}


/******************************************************************************
*
* CTrainingDlg::OnHelpContent()
*
* Prototype		:	void CTrainingDlg::OnHelpContent()
*
* Description	:	Load the help file
*
* In			:	None
*
* PreCond		:	None
*
* Out			:	None
*
* PostCond		:	None
*
* Remarks		:	None
*
******************************************************************************/
void CTrainingDlg::OnHelpContent() 
{

	if(LoadHelpFile("FC_TRAINING.exe") )
		::MessageBox(NULL,"Error in opening help file\nPlease verify that "HELP_FILE
		  " is available from application directory.","Help File", MB_ICONHAND | MB_OK);
}


/******************************************************************************
*
* OnDestroy
*
* Prototype		:	CTrainingDlg::OnDestroy
*
* Description	:	Called when WM_DESTROY is received, Call OnClose, close the
*					device and free allocated bitmap
*
* In			:	none
*
* PreCond		:	None
*
* Out			:	None
*
* PostCond		:	None
*
* Remarks		:	None
*
******************************************************************************/
void CTrainingDlg::OnDestroy() 
{
	CDialog::OnDestroy();
	
		//Stop current acquisition
		FC_StopAcquisition();

		//Stop acquisition thread
		OnClose();

		//Release mutex, event
		ReleaseMutex(hMutex);
		CloseHandle(hEvent);
		
		// Free the allocated memory
		if(pBitmap)
			delete pBitmap;	

		//Close the hardware 
		FC_CloseDevice();
	}



void CTrainingDlg::OnReadme() 
{
	if(LoadReadmeFile("FC_TRAINING.exe") )
		::MessageBox(NULL,"Error in opening Readme file\nPlease verify that "README_FILE
		  " is available from application directory.","Readme File", MB_ICONHAND | MB_OK);
}

LRESULT CTrainingDlg::WindowProc(UINT message, WPARAM wParam, LPARAM lParam) 
{
	// TODO: Add your specialized code here and/or call the base class
   	switch( message )
	{
	case WM_POWERBROADCAST:
		if(wParam == PBT_APMQUERYSUSPEND)
//		if((wParam == PBT_APMRESUMEAUTOMATIC) ||
//			(wParam == PBT_APMRESUMECRITICAL)|| (wParam ==PBT_APMRESUMESUSPEND)
//			|| (wParam ==PBT_APMQUERYSUSPENDFAILED ))
		{
			FC_StopAcquisition();
			killAcquisitionThread = true;
			SetEvent(hEvent);
			m_bReturnFromSuspend = TRUE ;
			//FC_CloseDevice() ;
		}
		break ;

	case WM_PAINT :
		if( m_bReturnFromSuspend )
		{
			Sleep(6000) ;
			if( 0 == hMutex )
				hMutex=CreateMutex(0,true,"");
			m_bReturnFromSuspend = FALSE ;
			killAcquisitionThread = FALSE ;
			AfxBeginThread(Threading, this);
		}
		break ;
	}

	return CDialog::WindowProc(message, wParam, lParam);
}
