/*******************************************************************************
*
*	Filename		:	FC_error.h
*	Description		:	define the constants and error codes
*						must be included everywhere
*
*	Company			:	ATMEL-Grenoble
*	Project			:	FingerChip
*	Authors			:	JFM Jean-Francois MAINGUET
*						YC  Yannick CHAMMING'S
*						JFD Jean-Francois DOREAU
*						GC  Gregory Clement
*						AC	Alain Chambron
*	Platform		:	independant
*	Language		:	C ANSI
*	Operating System:	independant
*
*	History
*	25-JUL-2002 JFM	:	v2.3 cosmetic changes
*	02-JUL-2003 AC	:	renamed FC_Error.h
*	28-JUL-2003 AC	:	merged with PC FC_Error
*						got rid of the bit-map idea
*	05-AUG-2003 AC	:	merged with SRD errors
*	20-AUG-2003 AC	:	added ErrCode definition
*						FC_FINGER_EXIXTS
*
*	Copyright (c)2001-2003  All rights reserved
******************************************************************************/

#ifndef __FC_ERROR__
#define __FC_ERROR__

// error codes are passed as shorts

typedef short ErrCode ;

#define	FC_OK					0

// ERROR

#define FC_ERROR_MASK				(short)(0x8000) 
#define FC_CODE_MASK				(short)(0x3fff)

// CRITICAL ERRORS

#define FC_ERROR					(short)(0x8000)	//Internal error
#define FC_DLL_NOT_INITIALIZED		(short)(0x8001)
#define FC_NOT_INITIALIZED			(short)(0x8002)
#define FC_NOT_OPEN					(short)(0x8003) //Device not opened
#define FC_NOT_CONNECTED			(short)(0x8004)
#define	FC_DRIVER_NOT_FOUND			(short)(0x8005) //driver may not be installed 
#define FC_DEVICE_NOT_CONNECTED		(short)(0x8006) //Device may be  not connected
#define FC_DRIVER_ERROR				(short)(0x8007) //Error during driver  acquisition
#define FC_BUSY						(short)(0x8008)
#define FC_OWNER					(short)(0x8009)	//-32765
#define FC_NOT_ENOUGH_MEMORY		(short)(0x800A)
#define	FC_NO_SLICE_MEMORY			(short)(0x800B) //no slice memory
#define	FC_NO_IMAGE_MEMORY			(short)(0x800C)
#define FC_NO_SYNCHRO				(short)(0x800D)
#define FC_SYNCHRO_LOST		 		(short)(0x800E)
#define FC_WRONG_SIZE_SLICE			(short)(0x800F) //wrong slice size 
#define FC_WRONG_IMAGE_WIDTH		(short)(0x8010)
#define FC_NO_BANDWIDTH				(short)(0x8011) //No bandwidth allocated for usb driver
#define FC_OPERATION_ABORTED 		(short)(0x8012) //Operation was aborted

// NON CRITICAL ERRORS

#define FC_BAD_FINGER				(short)(0xC001) //finger not swept fine
#define FC_IMAGE_TOO_BIG			(short)(0xC002)
#define	FC_IMAGE_TOO_SMALL			(short)(0xC003) //image is too small
#define FC_HEIGHT_TOO_LITTLE		(short)(0xC004)
#define FC_TOO_SLOW					(short)(0xC005)
#define FC_TOO_FAST					(short)(0xC006)
#define FC_NOT_ENOUGH_GOOD_SLICE	(short)(0xC007)
#define FC_TIME_OUT					(short)(0xC008)
#define FC_NOT_SUPPORTED            (short)(0xC009) //Parameter or function not 

// Enroll/Identify/Database Status Errors

#define FC_BAD_IMAGE				(short)(0xF001) //Enroll bad image, try again
#define FC_ENROLL_FAILED			(short)(0xF002) //Enroll command failed
#define FC_NO_MATCH					(short)(0xF003)
#define FC_NO_SUCH_FINGER			(short)(0xF004)
#define FC_DOWNLOAD_FAILED			(short)(0xF005) //Download database command failed
#define FC_UPLOAD_FAILED			(short)(0xF006) //Upload database command failed
#define FC_FINGER_EXISTS			(short)(0xF007)
//#define FC_BIO_NOT_OPENED			(short)(0xF007)
//#define FC_DB_NOT_OPENED			(short)(0xF008)


// WARNING

#define FC_DIRECTORY_MISSING		(short)(0x0001) //directory is missing
#define FC_UNABLE_OPEN_FILE			(short)(0x0002) //can't open the file
#define FC_UNCONSISTENT_SLICE		(short)(0x0004)
#define FC_NO_THERMAL				(short)(0x0005)
#define FC_THRESHOLD_HIGH			(short)(0x0006)
#define FC_HALF_BANDWIDTH			(short)(0x0007) //only half bandwidth allocated for usb driver
#define	FC_SLICE_MEMORY_FULL		(short)(0x0008)
#define FC_TEMPERATURE_NOT_UPTODATE	(short)(0x0009)
#define FC_TOO_WARM					(short)(0x000A)
#define FC_TOO_COLD					(short)(0x000B)
#define FC_SLOW						(short)(0x000C) //Sweeping was a bit slow
#define	FC_IMAGE_TRUNCATED			(short)(0x000D) //reconstructed image was truncated
#define FC_NO_FINGER				(short)(0x000E) //No finger detected
#define FC_TRUNCATED_MESSAGE		(short)(0x000F) 

// Enroll/Identify/Database Status Warnings
#define FC_GOOD_IMAGE				(short)(0x7001) //Enroll good image, other image(s) expected



#endif //__FC_ERROR__
